<?php
require_once 'PHPUnit/Util/Configuration.php';

PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');

/**
 * Wrapper for the PHPUnit XML configuration file.
 *
 * @author   Laurent Laville
 * @license  http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version  Release: 0.3.0a2
 * @since    Class available since Release 0.3.0
 */
class PEAR_TestListener_Configuration extends PHPUnit_Util_Configuration
{
    private static $instances = array();

    /**
     * Loads a PHPUnit configuration file.
     *
     * @param  string $filename
     */
    public function __construct($filename)
    {
        parent::__construct($filename);
    }

    /**
     * Returns a PHPUnit configuration object.
     *
     * @param  string $filename
     * @return PEAR_TestListener_Configuration
     */
    public static function getInstance($filename) {
        $realpath = realpath($filename);

        if ($realpath === FALSE) {
            throw new RuntimeException(
                sprintf('Could not read "%s".', $filename)
            );
        }

        if (!isset(self::$instances[$realpath])) {
            self::$instances[$realpath] = new PEAR_TestListener_Configuration($realpath);
        }
 
        return self::$instances[$realpath];
    }

    /**
     * Returns the logger configuration.
     *
     * @return array
     */
    public function getLoggerConfiguration()
    {
        $priorities = 'all|debug|info|notice|warning|error|critical|alert|emergency';
        $mPattern   = '/(MASK|MAX|MIN)?\s*'
                    .'('.$priorities.')+\s*'
                    .'(\&|\||\^)?\s*'
                    .'('.$priorities.')?'
                    .'/i';

        $result = array();

        foreach ($this->xpath->query('loggers') as $logger) {

            if ($logger->childNodes->item(1) instanceof DOMElement) {
                foreach ($logger->childNodes as $handler) {
                    if ($handler instanceof DOMElement) {
                        $_children = array();
                        $conf      = array();
                        switch ($handler->tagName) {
                        case 'composite' :
                            if ($handler->childNodes->item(1) instanceof DOMElement) {
                                foreach ($handler->childNodes as $child) {
                                    if ($child instanceof DOMElement) {
                                        $ident = (string)$child->getAttribute('ident');
                                        if (!empty($ident)) {
                                            $_children[$ident] = $child->tagName;
                                        }
                                    }
                                }
                            }
                            $ident = $name = $level = $mask = '';
                            break;
                        case 'file' :
                        case 'mail' :
                            $ident = (string)$handler->getAttribute('ident');
                            $name  = (string)$handler->getAttribute('name');
                            if (empty($ident) || empty($name)) {
                                // misconfiguring file handler, skip this entry
                                continue 2;
                            }
                            $level = (string)$handler->getAttribute('level');
                            $mask  = (string)$handler->getAttribute('mask');
                            if (preg_match($mPattern, $mask, $matches)) {
                                $mask = $this->logMaskAdapter($matches);
                                if ($mask === false) {
                                    // mask error, skip this entry
                                    continue 2;
                                }
                            }
                            break;
                        case 'null' :
                            $ident = (string)$handler->getAttribute('ident');
                            $name  = $level = '';
                            break;
                        }
                        
                        // common conf option for all handlers
                        if ($handler->childNodes->item(1) instanceof DOMElement) {
                            foreach ($handler->childNodes as $cfg) {
                                if ($cfg instanceof DOMElement) {
                                    if ($cfg->tagName == 'conf') {
                                        if ($cfg->childNodes->item(1) instanceof DOMElement) {
                                            foreach ($cfg->childNodes as $option) {
                                                if ($option instanceof DOMElement) {
                                                    $conf[$option->tagName] = $option->nodeValue;
                                                }
                                            }
                                        }  
                                    }
                                }
                            }
                        }

                        if (empty($level)) {
                            $level = 'debug';
                        }

                        $result[] = array(
                            'type'  => $handler->tagName,
                            'name'  => $name,
                            'ident' => $ident,
                            'conf'  => $conf,
                            'level' => $level,
                            'mask'  => $mask,
                            '_children' => $_children
                        );
                    }
                }
            }
        }
        return $result;
    }
    
    /**
     * Compute a complex mask
     * 
     * To compute the mask for a specific level, use for example:  MASK notice
     *     
     * To compute the mask for all priorities up to, and including error level, 
     * use:  MAX error     
     *  
     * To compute the mask for all priorities greater than or equal to a warning 
     * level, use:  MIN warning
     *
     * Masks can be be combined using bitwise operations like:  
     *  all ^ info
     *  info & notice
     *  error | warning 
     * 
     * @param array complex mask condition matches
     * 
     * @return mixed The current level mask, or FALSE on mask definition error
     */
    protected function logMaskAdapter($matches)
    {
        if (!empty($matches[1])) {
            // MASK, MIN or MAX 
            if (!empty($matches[3]) || !empty($matches[4])) {
                // mask error
                return false;
            }
            $minMAX = strtoupper($matches[1]);
            $priority = Log::stringToPriority(strtolower($matches[2]));
            $mask = Log::$minMAX($priority);
        } elseif (!empty($matches[3])) {
            // bitwise operator 
            if (empty($matches[2]) || empty($matches[4])) {
                // mask error
                return false;
            }
            if ($matches[2] == 'all') {
                $p1 = PEAR_LOG_ALL;
            } else {
                $p1 = Log::stringToPriority(strtolower($matches[2]));
                $p1 = Log::MASK($p1);
            }
            if ($matches[4] == 'all') {
                $p2 = PEAR_LOG_ALL;
            } else {
                $p2 = Log::stringToPriority(strtolower($matches[4]));
                $p2 = Log::MASK($p2);
            }
            switch ($matches[3]) {
            case '^':
                $mask = $p1 ^ $p2;
                break;
            case '&':
                $mask = $p1 & $p2;
                break;
            case '|':
                $mask = $p1 | $p2;
                break;
            }
        } else {
            // mask error
            return false;
        }
        $mask = dechex($mask);
        return $mask;
    }
}
?>